#!/usr/bin/perl
# 
# sb::Plugin::TemplateMaker - Module for sb
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

# 0.02 [2006/11/03] implemented opened form feature / changed bracket rule
# 0.01 [2006/08/03] changed detailt to point new site address
# 0.00 [2006/02/13] created

package sb::Plugin::TemplateMaker;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin',
		'name'    => 'Make Template',
		'text'    => 'template maker',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => '0.02',
	},
	'file' => 'templatemaker.txt',
	'data' => 1,
);
sb::Plugin->register_admin_module(
	'mode'   => 'maketemp',
	'level'  => 0,
	'module' => 'sb::Admin::TemplateMaker',
);
package sb::Admin::TemplateMaker;
# ==================================================
# // declaration for global variables
# ==================================================
use strict;
use vars qw( @ISA );
# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Config ();
use sb::Language ();
use sb::Interface ();
use sb::Data::Image ();
use sb::Admin::Template ();
@ISA = qw( sb::Admin::Template );
# ==================================================
# // declaration for constant value
# ==================================================
sub TEMPLATE        (){ 'templatemaker.html' };
sub DEFAULT_TYPE    (){ 'editor' };
sub MAX_UPLOAD_NUM  (){ 5 };
sub EXTRA_CSSPREFIX (){ 'ext_' };
sub EXTRA_STYLE     (){ '<link rel="stylesheet" type="text/css" href="%s" title="default" media="screen,tv" />' };
sub ADDITIONAL_CSS  (){ "<style type=\"text/css\">\n%s\n</style>" };
sub SEPARATOR       (){ '===sbTempPack' };
sub FORMAT_SEP      (){ 'From ???@??? %Week% %MonShort% %Day% %Hour%:%Min%:%Sec% %Year%' };
sub FORMAT_DATE     (){ '%Week%, %Day% %MonShort% %Year% %Hour%:%Min%:%Sec% %Zone%' };
sub EXPORT_CHARSET  (){ 'ISO-2022-JP' };
sub EXPORT_CODE     (){ 'jis' };
# ==================================================
# // public functions - callback
# ==================================================
sub callback
{
	my $self = shift;
	$self->{'temp'} = undef;
	$self->{'data'} = sb::Plugin->get_data;
	return ( $self->{'regi'} ) 
		? $self->_update_template(@_)
		: $self->_open_template(@_);
}
# ==================================================
# // public functions - override
# ==================================================
sub common_template_parts
{
	my $self = shift;
	my $cms = shift;
	my $lang = sb::Language->get;
	return if (!$cms or !$self);
	$self->SUPER::common_template_parts($cms);
	my $css = '';
	if ($self->{'user'} and $self->{'user'}->ad_css ne '')
	{
		my $ext = sb::Config->get->value('srv_temp') . $lang->code . '/' . EXTRA_CSSPREFIX;
		$css .= "\n" . sprintf(EXTRA_STYLE,$ext . $self->{'user'}->ad_css . '.css');
	}
	$css .= "\n" . sprintf(ADDITIONAL_CSS,$lang->string('parts_maketempExtraStyle'));
	$cms->num(0);
	$cms->tag('sb_extra_style'=>$css);
}
# ==================================================
# // private functions - main routine
# ==================================================
sub _update_template
{
	my $self = shift;
	my %param = ('message'=>'',@_);
	my $cgi  = sb::Interface->get;
	my $type = $self->_check_type($cgi->value('__type'));
	my $lang = sb::Language->get;
	my $msg  = '';
	SWITCH_REGI: {
		$_ = $self->{'regi'};
		/^change$|^select$/ && do
		{
			if ($self->{'regi'} eq 'select')
			{
				$self->{'data'}->setting('');
				$self->{'temp'} = undef;
				$msg = $lang->string('parts_maketempReset');
			}
			elsif ($cgi->value('select') eq 'new')
			{
				my $new = sb::Data->add('Template',
					'use'   => 0,
					'name'  => $lang->string('parts_noname'),
					'gen'   => $self->{'time'},
					'mod'   => $self->{'time'},
					'main'  => '',
					'css'   => '',
					'entry' => '',
					'info'  => '',
				);
				if ($new)
				{
					sb::Data->update($new);
					$self->{'temp'} = $new;
					$self->{'data'}->setting($new->id);
				}
				$msg = $lang->string('parts_maketempCreated');
			}
			else
			{
				$self->{'data'}->setting($cgi->value('select'));
				$msg = $lang->string('parts_maketempSelected');
			}
			sb::Plugin->set_data('data'=>$self->{'data'});
			last SWITCH_REGI;
		}; # end of change|select
		/^rename$/ && do
		{
			last SWITCH_REGI if (!$self->{'temp'});
			my $info = $self->_parse_info;
			my $name = sb::Text->entitize($cgi->value('template_name'));
			$name = $lang->string('parts_noname') if ($name eq '');
			$self->{'temp'}->name($name);
			$info->{'subj'} = sb::Text->detitize($name);
			$info->{'from'} = $cgi->value('template_author');
			$info->{'addr'} = $cgi->value('template_webpage');
			$info->{'vers'} = $cgi->value('template_version');
			$self->{'temp'}->info($self->_create_info($info,$cgi->value('template_info')));
			sb::Data->update($self->{'temp'});
			$msg = $lang->string('parts_maketempRenamed');
			last SWITCH_REGI;
		}; # end of rename
		/^update$/ && do
		{
			last SWITCH_REGI if (!$self->{'temp'});
			$self->{'temp'}->main($cgi->value('template_main'));
			$self->{'temp'}->css($cgi->value('template_css'));
			$self->{'temp'}->entry($cgi->value('template_entry'));
			sb::Data->update($self->{'temp'});
			$msg = $lang->string('parts_maketempModified');
			last SWITCH_REGI;
		}; # end of update
		/^add$/ && do
		{
			last SWITCH_REGI if (!$self->{'temp'});
			my @list = split(',',$self->{'temp'}->files);
			my @news = $self->_upload($cgi);
			push(@list,@news) if (@news);
			$self->{'temp'}->files(join(',',@list));
			sb::Data->update($self->{'temp'});
			$msg = sprintf($lang->string('parts_add_comp'),$#news + 1);
			last SWITCH_REGI;
		}; # end of add
		/^delete$/ && do
		{
			last SWITCH_REGI if (!$self->{'temp'});
			my $num = 0;
			my $dir = sb::Config->get->value('conf_dir_base') . sb::Config->get->value('dir_style');
			my @sels = split("\0",$cgi->value('sel'));
			my @list = split(',',$self->{'temp'}->files);
			foreach my $part (@sels)
			{
				my $file = $dir . $part;
				next if (!-e $file);
				unlink($file);
				@list = grep(!/^$part$/,@list);
				$num++;
			}
			$self->{'temp'}->files(join(',',@list));
			sb::Data->update($self->{'temp'});
			$msg = $num . $lang->string('parts_deleted');
			last SWITCH_REGI;
		}; # end of delete
	}; # end of SWITCH_REGI
	$msg = $lang->string('error_unknown') if ($msg eq '');
	return $self->_open_template('message'=>$msg);
}
sub _open_template
{
	my $self = shift;
	my %param = ('message'=>'',@_);
	my $cgi = sb::Interface->get;
	my $cms = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $type = $self->_check_type($cgi->value('__type'));
	my $temp = $self->{'temp'};
	if ($type eq 'export' and $cgi->value('export') eq 'on')
	{
		require 'mimeutil.pl';
		return sb::Interface->get->head('type'=>'text/plain','charset'=>EXPORT_CHARSET) . $self->_export();
	}
	else
	{
		SWITCH_TYPE: {
			$_ = $type;
			/^editor$/ && do
			{
				if ($temp)
				{
					my $viewpath = sb::Config->get->value('conf_srv_cgi') . sb::Config->get->value('basic_sb');
					$cms->num(0);
					$cms->tag('sb_maketemp_viewlink'=>$viewpath . '?tid=' . $temp->id);
					$cms->tag('sb_temp_main'=>sb::Text->entitize($temp->main));
					$cms->tag('sb_temp_css'=>sb::Text->entitize($temp->css));
					$cms->tag('sb_temp_entry'=>sb::Text->entitize($temp->entry));
					if ($temp->files ne '')
					{
						my $selector = '';
						my @parts = split(',',$temp->files);
						foreach my $part (@parts)
						{
							$selector .= '<option value="{site_parts}' . $part . '">' . $part . '</option>' . "\n"
						}
						$cms->tag('sb_maketemp_image'=>$selector);
					} # end of if ($temp->files ne '')
					my $opened = $cgi->value('openedform');
					$opened = 'main' if ($opened ne 'css' and $opened ne 'entry');
					$cms->tag('sb_maketemp_open'=>$opened);
				}
				else
				{
					my @tmps = sb::Data->load('Template','sort'=>'mod','order'=>1);
					my $template_selector = '';
					foreach my $tmp (@tmps)
					{
						next if ($tmp->use);
						$template_selector .= '<option value="' . $tmp->id . '">' . $tmp->name . '</option>' . "\n";
					}
					$cms->num(0);
					$cms->tag('sb_maketemp_selector'=>$template_selector);
				}
				last SWITCH_TYPE;
			}; # end of editor
			/^info$/ && do
			{
				my $info = $self->_parse_info;
				$cms->num(0);
				$cms->tag('sb_temp_info'=>sb::Text->entitize($info->{'body'}));
				$cms->tag('sb_temp_name'=>$temp->name);
				$cms->tag('sb_temp_author'=>sb::Text->entitize($info->{'from'}));
				$cms->tag('sb_temp_url'=>sb::Text->entitize($info->{'addr'}));
				$cms->tag('sb_temp_version'=>sb::Text->entitize($info->{'vers'}));
				last SWITCH_TYPE;
			}; # end of info
			/^upload$/ && do
			{
				for (my $i=0;$i<MAX_UPLOAD_NUM;$i++)
				{
					$cms->num($i);
					$cms->tag('sb_upload_num'=>$i);
				}
				$cms->block('sb_maketemp_upload_file'=>MAX_UPLOAD_NUM);
				if ($temp->files ne '')
				{
					my $num = 0;
					my @parts = split(',',$temp->files);
					foreach my $part (@parts)
					{
						my $check = '<input type="checkbox" name="sel" value="' . $part . '" onclick="switchList(this)" />';
						$cms->num($num);
						$cms->tag('sb_list_class'=>($num % 2) ? 'odd' : 'even');
						$cms->tag('sb_img_name'=>$part);
						$cms->tag('sb_img_sel'=>$check);
						$num++;
					}
					$cms->block('sb_maketemp_upload_list'=>$num);
				} # end of if ($temp->files ne '')
				last SWITCH_TYPE;
			}; # end of upload
		}; # end of SWITCH_TYPE
		$cms->num(0);
		$cms->tag('sb_maketemp_menu_' . $type => 'class="current"');
		if (!$temp)
		{
			$cms->block('sb_maketemp_editor'=>0);
			$cms->block('sb_maketemp_select'=>1);
		}
		else
		{
			$cms->block('sb_maketemp_' . $type => 1);
		}
		if ($param{'message'} ne '')
		{
			$cms->num(0);
			$cms->tag('sb_process_message'=>$param{'message'});
			$cms->block('sb_process_message'=>1);
		}
		$self->common_template_parts($cms);
		return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
	}
}
# ==================================================
# // private functions - utilities
# ==================================================
sub _check_type
{
	my $self = shift;
	my $type = shift;
	$type = DEFAULT_TYPE if ($type eq '');
	if ($self->{'data'}->setting ne '' and !$self->{'temp'})
	{
		$self->{'temp'} = sb::Data->load('Template','id'=>$self->{'data'}->setting);
	}
	$type = DEFAULT_TYPE if (!$self->{'temp'});
	return $type;
}
sub _upload
{
	my $self = shift;
	my $cgi = shift;
	my $dir = sb::Config->get->value('conf_dir_base') . sb::Config->get->value('dir_style');
	my @forms = ();
	my @files = ();
	for (my $i=0;$i<MAX_UPLOAD_NUM;$i++)
	{
		push(@forms,$i) if ($cgi->value('upload_file' . $i) ne '');
	}
	my @header = $cgi->content_list;
	for (my $i=0;$i<=$#forms;$i++)
	{
		my $entity = $cgi->value('upload_file' . $forms[$i]);
		my $label  = 'upload_file' . $forms[$i];
		my %param  = ();
		foreach (@header)
		{
			next if ($_ !~ /$label/);
			$param{'type'}     = $2 if ($_ =~ /(.*)Content-type:(.*)/i);
			$param{'filename'} = $2 if ($_ =~ /(.*)filename=(.*)/i);
			$param{'macbin'}   = ($_ =~ /application\/x-macbinary/i) ? 1 : undef;
			last;
		}
		if ($param{'macbin'})
		{ # handle mac binary
			my $len = substr($entity,83,4);
			$len = unpack("%N",$len);
			$entity = substr($entity,128,$len);
		}
		$param{'type'} =~ tr/\x0D\x0A//d;
		$param{'type'} =~ s/\s//g;
		$param{'filename'} =~ tr/\"\x0D\x0A//d;
		$param{'filename'} =~ s/.*[:\/\\](.*)/$1/; # extract file name only
		next if ($param{'filename'} =~ /[^a-zA-Z0-9_\-\.]/);
		next if (-e $dir . $param{'filename'});
		my $file = $dir . $param{'filename'};
		eval
		{
			open(BINOUT,">$file") or die('failed file open');
			binmode(BINOUT);
			print BINOUT $entity;
			close(BINOUT);
			chmod(sb::Config->get->value('basic_file_attr'),$file);
		};
		next if ($@);
		push(@files,$param{'filename'});
	}
	return(@files);
}
sub _export
{
	my $self = shift;
	my $sep = SEPARATOR . $self->{'time'} . $$ . '===';
	my @body = $self->_get_info();
	$body[0] .= 'Content-Type: multipart/mixed; boundary="' . $sep . '"' . "\n";
	$body[0] .= "\n"; # separator between header and body
	$body[0] .= 'This is a multi-part message in MIME format.' . "\n";
	push(@body,$self->_get_as_mime('main'));
	push(@body,$self->_get_as_mime('css'));
	push(@body,$self->_get_as_mime('entry')) if ($self->{'temp'}->entry ne '');
	my @files = split(',',$self->{'temp'}->files);
	my $dir = sb::Config->get->value('conf_dir_base') . sb::Config->get->value('dir_style');
	foreach my $file (@files)
	{
		next if (!-r $dir . $file);
		push(@body,$self->_add_parts($dir . $file));
	}
	my $output = '';
	for (my $i=0;$i<@body;$i++)
	{
		$output .= $body[$i] . '--' . $sep;
		$output .= ($i == $#body) ? '--' . "\n" . '.' . "\n" : "\n";
	}
	return $output;
}
sub _create_info
{
	my $self = shift;
	my $info = shift;
	my $body = shift;
	my $out = '';
	$out .= 'Name: ' . $info->{'subj'} . "\n" if ($info->{'subj'} ne '');
	$out .= 'Author: ' . $info->{'from'} . "\n" if ($info->{'from'} ne '');
	$out .= 'Address: ' . $info->{'addr'} . "\n" if ($info->{'addr'} ne '');
	$out .= 'Version: ' . $info->{'vers'} . "\n" if ($info->{'vers'} ne '');
	$out .= "=====\n" if ($out ne '' and $body ne '');
	$out .= $body;
	return($out);
}
sub _parse_info
{
	my $self = shift;
	my $conv = shift;
	my $out  = {};
	my ($head,$body) = split("=====\n",$self->{'temp'}->info,2);
	my @info = split("\n",$head);
	foreach my $line (@info)
	{
		$out->{'from'} = $1 if ($line =~ /^\s*Author:\s+(.*)\s*$/i);
		# $out->{'subj'} = $1 if ($line =~ /^\s*Name:\s+(.*)\s*$/i); # not in use
		$out->{'vers'} = $1 if ($line =~ /^\s*Version:\s+(.*)\s*$/i);
		$out->{'addr'} = $1 if ($line =~ /^\s*Address:\s+(.*)\s*$/i);
	}
	$out->{'subj'} = sb::Text->detitize($self->{'temp'}->name);
	$out->{'body'} = '';
	if ($conv)
	{
		my @lines = split("\n",$body);
		foreach my $line (@lines)
		{
			$line =~ s/^\./\.\./;
			$out->{'body'} .= $line . "\n";
		}
	}
	else
	{
		$out->{'body'} = $body;
	}
	return($out);
}
# ==================================================
# // private functions - for export
# ==================================================
sub _get_info
{
	my $self = shift;
	my $out  = $self->_parse_info('convert_body');
	my $lang = sb::Language->get;
	$out->{'body'} = $self->_create_info($out,$out->{'body'});
	$out->{'from'} .= ' <' . $out->{'addr'} . '>' if ($out->{'addr'});
	$out->{'head'}  = sb::Time->format('time'=>$self->{'time'},'form'=>FORMAT_SEP) . "\n";
	$out->{'head'} .= 'To: sb users' . "\n";
	$out->{'head'} .= 'From: ' . $out->{'from'} . "\n";
	$out->{'head'} .= 'Date: ' . sb::Time->format('time'=>$self->{'time'},'form'=>FORMAT_DATE) . "\n";
	$out->{'head'} .= 'Subject: ' . $out->{'subj'} . "\n";
	$out->{'head'} .= 'X-Version: ' . $out->{'vers'} . "\n";
	if ($lang->charcode ne EXPORT_CODE)
	{
		$lang->checkcode('',$lang->charcode);
		$out->{'head'} = $lang->convert($out->{'head'},EXPORT_CODE);
	}
	$out->{'head'} = &mimeutil::mimeencode($out->{'head'});
	if ($lang->charcode ne EXPORT_CODE)
	{
		$lang->checkcode('',$lang->charcode);
		$out->{'body'} = $lang->convert($out->{'body'},EXPORT_CODE);
	}
	my $outbody = 'Contet-Type: text/plain; charset=' . EXPORT_CHARSET . "\n";
	$outbody .= 'Content-Transfer-Encoding: 7bit' . "\n\n";
	$outbody .= $out->{'body'};
	return($out->{'head'},$outbody);
}
sub _get_as_mime
{
	my $self = shift;
	my $type = shift;
	my %info = (
		'main'  => {'name'=>'base.html', 'type'=>'text/html'},
		'css'   => {'name'=>'style.css', 'type'=>'text/css'},
		'entry' => {'name'=>'entry.html','type'=>'text/html'},
	);
	my $out = '';
	$out .= 'Content-Type: ' . $info{$type}->{'type'} . '; name="' . $info{$type}->{'name'} . '"' . "\n";
	$out .= 'Content-Transfer-Encoding: Base64' . "\n";
	$out .= 'Content-Disposition: attachment; filename="' . $info{$type}->{'name'} . '"' . "\n";
	$out .= "\n";
	$out .= &mimeutil::bodyencode($self->{'temp'}->$type(),'b64');
	$out .= &mimeutil::benflush('b64');
	return($out);
}
sub _add_parts
{
	my $self = shift;
	my $file = shift;
	my $out = '';
	my $len = sb::Config->get->value('basic_base_enc');
	my $name = $file;
	my $type = &sb::Data::Image::_file_contenttype($name);
	$name =~ s/.*[:\/\\](.*)/$1/; # extract file name only
	$out .= 'Content-Type: ' . $type . '; name="' . $name . '"' . "\n";
	$out .= 'Content-Transfer-Encoding: Base64' . "\n";
	$out .= 'Content-Disposition: attachment; filename="' . $name . '"' . "\n";
	$out .= "\n";
	open(IN, $file);
	binmode IN;
	while( read(IN, $_, $len) )
	{
		$out .= &mimeutil::bodyencode($_,'b64');
	}
	$out .= &mimeutil::benflush('b64');
	return($out);
}
1;
__END__
